/* Michael Olson's tweaked Muse stylesheet

   Elements for most of my web site.

   License: Modification and propagation of this document is
            permitted.  Attribution of credit to me is optional.
*/

/* Table of Contents */

.contents {
  clear: left;
  float: left;
  background-color: #D3D3D3;    /* light gray */
  border: 1px solid black;
  margin-top: 0;
  margin-right: 5px;
  margin-left: 0;
  margin-bottom: 15px;
  padding: 3px;
}

.contents dl {
  background-color: #F5F5F5;    /* white smoke */
  display: block;
  float: left;
  font-size: small;
  margin-top: 0;
  margin-bottom: 0;
  padding-bottom: 5px;
}

.contents dt {
  clear: left;
  display: block;
  padding-right: 5px;
  padding-left: 5px;
  text-align: left;
}

.contents dd dt {
  display: list-item;
}

.contents a:hover { 
  background-color: #D3D3D3;    /* light gray */
}

/* Planner */

.taskA { color: red }

.taskB { color: green }

.taskC { color: navy }

.taskX {
  color: #696969;               /* dim gray */
  text-decoration: line-through;
}

/* Source coloring */

.builtin {
  /* font-lock-builtin-face */
  color: #9370db;
}

.comment {
  /* font-lock-comment-face */
  color: #b22222;
}

.constant {
  /* font-lock-constant-face */
  color: #b8860b;
}

.doc {
}

.function-name {
  /* font-lock-function-name-face */
  color: #0000ff;
}

.keyword {
  /* font-lock-keyword-face */
  color: #a020f0;
}
  
.string {
  /* font-lock-string-face */
  color: #228b22;
}
