<?xml version="1.0" encoding="<lisp>
  (muse-html-encoding)</lisp>"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
  <title><lisp>(concat (car (muse-project))
                       " - "
                       (muse-wiki-publish-pretty-title))</lisp></title>
  <link rel="alternate" type="application/rss+xml" title="RSS" href="http://blog.mwolson.org/index.rss" />
  <link rel="author" href="http://mwolson.org/web/AboutMe.html" />
  <link rel="home" href="http://mwolson.org/web/WelcomePage.html" />
  <link rel="shortcut icon"
        href="http://mwolson.org/static/logos/favicon.ico" />
  <lisp>
    (muse-style-element :style-sheet muse-publishing-current-style)
  </lisp>
  <meta name="description" content="Michael Olson, maintainer of Emacs Muse and ERC" />
  <meta name="keywords" content="emacs,muse,erc,emms" />
  <meta name="generator" content="Muse" />
  <meta http-equiv="<lisp>muse-html-meta-http-equiv</lisp>" content="<lisp>muse-html-meta-content-type</lisp>" />
</head>

<body>

<div class="menu">
  <div class="menuitem">
    <a title="About me, contact info." href="/web/AboutMe.html">About me</a>
  </div>

  <div class="menuitem">
    <a title="Photo albums and the like." href="http://mwolson.risnerolson.org/albums/">Albums</a>
  </div>

  <div class="menuitem">
    <a title="My blog." href="http://blog.mwolson.org/">Blog</a>
  </div>

  <div class="menuitem">
    <a title="Bookmarks from my web browser." href="/bookmarks/">Bookmarks</a>
  </div>

  <div class="menuitem">
    <a title="Stuff from some of my classes." href="/notes/WikiIndex.html">Notes</a>
  </div>

  <div class="menuitem">
  <a title="Philisophical writings and ideas." href="/web/PhilosophyPage.html">Philosophy</a>
  </div>

  <div class="menuitem">
    <a title="Software that I work on, configuration." href="/projects/">Projects</a>
  </div>

  <div class="menuitem">
    <a title="Miscellaneous things I work on." href="/web/WorksPage.html">Works</a>
  </div>

</div><!-- menu ends here -->

<h1 id="top">
<a href="<lisp>(muse-wiki-resolve-project-page "Website")</lisp>"
style="text-decoration: none;">
<img src="/static/logos/site-logo.png"
alt="mwolson.org logo" />
</a>
<a class="top" href="<lisp>(muse-wiki-resolve-project-page
                             (muse-project))</lisp>">
<lisp>(concat (car (muse-project)) " - "
              (muse-wiki-publish-pretty-title))</lisp>
</a>
</h1>

<!-- Page published by Emacs Muse begins here -->
